/*
Copyright (c) 2005, MentorGen, LLC
All rights reserved.

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions are met:

+ Redistributions of source code must retain the above copyright notice, 
  this list of conditions and the following disclaimer.
+ Redistributions in binary form must reproduce the above copyright notice, 
  this list of conditions and the following disclaimer in the documentation 
  and/or other materials provided with the distribution.
+ Neither the name of MentorGen LLC nor the names of its contributors may be 
  used to endorse or promote products derived from this software without 
  specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
  POSSIBILITY OF SUCH DAMAGE.
 */
package com.mentorgen.tools.profile.instrument;

import static org.objectweb.asm.jip.Opcodes.INVOKESTATIC;

import org.objectweb.asm.jip.ClassAdapter;
import org.objectweb.asm.jip.ClassVisitor;
import org.objectweb.asm.jip.FieldVisitor;
import org.objectweb.asm.jip.Label;
import org.objectweb.asm.jip.MethodVisitor;
import org.objectweb.asm.jip.Opcodes;

import com.mentorgen.tools.profile.Controller;

import em.jop.Profiler;

/**
 * 
 * @author Andrew Wilcox
 * @see org.objectweb.asm.jip.ClassAdapter
 */
public class PerfClassAdapter extends ClassAdapter {
	private String className;
	
	//EM 20/10/2008
	String superName;
	boolean finalizeViewed = false;
	boolean isInterface = false;
	
	public PerfClassAdapter(ClassVisitor visitor, String theClass) {
		super(visitor);
		this.className = theClass;
		//EM 16/10/2008
//		System.out.println("PerfClassAdapter:"+className);
	}
	
	//EM 20/10/2008
	@Override
	public void visit(int version, int access, String name, String signature,
			String superName, String[] interfaces) {
		//EM 20/10/2008
		this.superName = superName; 
		if((access & Opcodes.ACC_INTERFACE) != 0){
			isInterface = true;
		}
		
		if(superName.equals("java/lang/Thread")){
			System.out.println("JOPed inheritance on "+name+" Thread -> JOPThread");
			super.visit(version, access, name, signature, "em/jop/profiler/JOPThread", interfaces);
		}
		else{
			super.visit(version, access, name, signature, superName, interfaces);
		}
	}

	public MethodVisitor visitMethod(int arg,
			String name,
			String descriptor,
			String signature,
			String[] exceptions) {
		//EM 20/10/2008 : search for finalize call
		//System.out.println("METH:" + className + "." + name + ":" + descriptor);
		if(name.equals("finalize")){
			//System.out.println("FINALIZE FOUND:" + className);
			finalizeViewed = true;
		}
		
		MethodVisitor mv = super.visitMethod(arg, 
				name, 
				descriptor, 
				signature, 
				exceptions);
		if (Controller._outputMethodSignatures && descriptor != null) {
			//EM 22/10/2008 : also need superName
			return new PerfMethodAdapter(mv, className, name + descriptor,superName);
		} else {
			//EM 22/10/2008 : also need superName
			return new PerfMethodAdapter(mv, className, name,superName);
		}
	}

	//EM 20/10/2008
	@Override
	public FieldVisitor visitField(int access, String name, String desc,
			String signature, Object value) {
		if(desc.equals("java/lang/Thread")){
			//System.out.println("JOPed field "+name+" Thread -> JOPThread");
			return super.visitField(access, name, "em/jop/profiler/JOPThread", signature, value);
		}
		else{
			return super.visitField(access, name, desc, signature, value);
		}
	}

	//EM 20/10/2008 : add finalize to all not finalized classes
	@Override
	public void visitEnd() {
		if(!finalizeViewed && !isInterface){
//			System.out.println("ADD FINALIZE:" + className);
			
			MethodVisitor mv = super.visitMethod(Opcodes.ACC_PROTECTED, "finalize", "()V", null, new String[] { "java/lang/Throwable" });
			mv.visitCode();


			Label l0 = new Label();
			mv.visitLabel(l0);
			mv.visitVarInsn(Opcodes.ALOAD, 0);
			mv.visitLdcInsn(className.replaceAll("/", "."));
			mv.visitMethodInsn(INVOKESTATIC, 
					Controller._profiler, 
					"gc", 
					"(Ljava/lang/Object;Ljava/lang/String;)V");			

			Label l1 = new Label();
			mv.visitLabel(l1);
			mv.visitVarInsn(Opcodes.ALOAD, 0);
			mv.visitMethodInsn(Opcodes.INVOKESPECIAL, superName, "finalize", "()V");
			Label l2 = new Label();
			mv.visitLabel(l2);
			mv.visitInsn(Opcodes.RETURN);
			Label l3 = new Label();
			mv.visitLabel(l3);
			mv.visitLocalVariable("this", "L"+className+";", null, l0, l3, 0);
			mv.visitMaxs(2, 1);
			mv.visitEnd();
		}
		// TODO Auto-generated method stub
		super.visitEnd();
	}

//	@Override
//	protected void finalize() throws Throwable {
//		Profiler.gc(this, "className");
//		// TODO Auto-generated method stub
//		super.finalize();
//	}

}
