package em.jop;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.GraphicsConfiguration;
import java.awt.HeadlessException;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.geom.Rectangle2D;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.Socket;
import java.text.NumberFormat;
import java.util.Comparator;
import java.util.Date;
import java.util.TreeMap;
import java.util.TreeSet;
import java.util.Vector;

import javax.imageio.ImageIO;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.border.Border;

import em.jop.console.AllocView;
import em.jop.console.Client;
import em.jop.console.Colors;
import em.jop.console.MethodView;
import em.jop.console.ThreadStackView;

public final class Console extends JFrame {
	static final public String _TITLE = "JOP Console 0.4.1";
	static final int _LINEHEIGHT = 15;
	//EM 28/10/2008
	static final int _SHOWTREE_THREAD = 0;
	static final int _SHOWTREE_METHOD = 1;
	static final int _SHOWTREE_CALLEE = 2;
	static final int _SHOWTREE_ALLOC = 3;
	
	//EM 29/10/2008 : first data/interface separation
	Client client = null;
	
	//EM 14/10/2008 : main interface objects
	Image logoI = null;
	//EM 26/10/2008
	Image backgroundI = null;

	JPanel treeView;
	int treeMaxXPos = 0;
	int treeYPos = 0;
	NumberFormat nf = NumberFormat.getInstance();
	{
		//nf.setMinimumIntegerDigits(2);
		nf.setMinimumFractionDigits(2);
		nf.setMaximumFractionDigits(2);
	}

	//EM 29/10/2008
	Font stdFont = new Font("monospaced", Font.PLAIN, 14);
	Font boldFont = new Font("monospaced", Font.BOLD, 14);
	Font itFont = new Font("monospaced", Font.ITALIC, 14);
	Font smallFont = new Font("monospaced", Font.PLAIN, 10);
	Font smallItFont = new Font("monospaced", Font.ITALIC, 10);
	
	public JLabel statusL;
	//EM 28/10/2008
	public JTextField serverIP;

	//EM 18/10/2008
	JComboBox viewSelectorCB;

	//EM 12/10/2008 : mem viewing
	JPanel memView;

	public Console() throws HeadlessException {
		this(_TITLE);
		
		//EM 29/10/2008 : first data/interface separation
		client = new Client(this);

		//EM 14/10/2008
		try{
			logoI = ImageIO.read(this.getClass().getResourceAsStream("img/Logo.png"));
			//EM 26/10/2008
			backgroundI = ImageIO.read(this.getClass().getResourceAsStream("img/Background.png"));
		}
		catch(Throwable t){
			//??
		}

		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		JPanel aButPanel = new JPanel();
		aButPanel.setLayout(new FlowLayout());
		getContentPane().add(aButPanel,BorderLayout.NORTH);
		//EM 18/10/2008 : view selector
		aButPanel.add(viewSelectorCB = new JComboBox());
		viewSelectorCB.addItem("Thread tree");
		viewSelectorCB.addItem("Method tree");
		viewSelectorCB.addItem("Callee tree");
		viewSelectorCB.addItem("Alloc tree");
		JButton aRefreshB = new JButton("Refresh");
		aButPanel.add(aRefreshB);
		JButton aSetMarkB = new JButton("Set mark");
		aButPanel.add(aSetMarkB);
		final JCheckBox aDispFromMarkCB = new JCheckBox("Display from mark",client.dispFromMark);
		aButPanel.add(aDispFromMarkCB);
		final JCheckBox aDispJopTimesCB = new JCheckBox("Display JOP overhead",client.dispJopTimes);
		aButPanel.add(aDispJopTimesCB);
		//EM 21/01/2008
		JButton aForceGcB = new JButton("Force GC");
		aButPanel.add(aForceGcB);

		treeView = new JPanel(){
			protected void paintComponent(Graphics g) {
				paintTree(g);
			}
		};
		treeView.addMouseListener(new MouseAdapter(){
			public void mouseClicked(MouseEvent arg0) {
				int aLineIndex = arg0.getY() / _LINEHEIGHT;
				if(client.showTree == _SHOWTREE_ALLOC && aLineIndex < client.displayAllocsNb){
					client.displayAllocs[aLineIndex].isExpended = !client.displayAllocs[aLineIndex].isExpended;
					if(client.displayAllocs[aLineIndex].isExpended){
						//Populate subs
						AllocView aA = client.displayAllocs[aLineIndex];
						while(true){
							Object[] aMs = aA.locationSorted.toArray();
							for(int i = 0;i < aMs.length;i++){
								((AllocView)aMs[i]).buildCalleePopulate(client.calleesTreeRoot, client.compAllocs);
							}
							if(aMs.length != 1){
								break;
							}
							aA = (AllocView)aMs[0];
							aA.isExpended = true;
						}
					}
				}
				else if((client.showTree == _SHOWTREE_METHOD //
						|| client.showTree == _SHOWTREE_THREAD //
						|| client.showTree == _SHOWTREE_CALLEE) //
						&& aLineIndex < client.displayMethodsNb){
					client.displayMethods[aLineIndex].isExpended = !client.displayMethods[aLineIndex].isExpended;
					if(client.displayMethods[aLineIndex].isExpended){
						//Expend significative subs
						MethodView aM = client.displayMethods[aLineIndex];

						while(true){
							if(aM.subMethodsSorted == null){
								break;
							}
							if(client.showTree == _SHOWTREE_CALLEE){
								//Populate subs
								Object[] aMs = aM.subMethodsSorted.toArray();
								for(int i = 0;i < aMs.length;i++){
									((MethodView)aMs[i]).buildCalleePopulate(client.calleesTreeRoot, client.compTimeSpent);
								}
							}
							//EM 18/10/2008
							if(client.showTree == _SHOWTREE_CALLEE && aM.subMethodsSorted.size() > 1){
								break;
							}
							try{
								aM = aM.subMethodsSorted.first();
							}
							catch(Throwable t){
								//No first
								break;
							}
							if(aM == null){
								break;
							}
							if(client.showTree == 0 //
									&& aM.timeSpent < client.displayMethods[aLineIndex].timeSpent * 0.90){
								break;
							}
							aM.isExpended = true;
						}
					}
				}
				treeView.repaint();
			}

		});
		JScrollPane aSP = new JScrollPane();
		aSP.getViewport().add(treeView);
		aSP.getVerticalScrollBar().setUnitIncrement(15);

		JPanel aCenterP = new JPanel();
		aCenterP.setLayout(new BorderLayout());
		aCenterP.add(aSP,BorderLayout.CENTER);

		memView = new JPanel(){
			protected void paintComponent(Graphics g) {
				paintMem(g);
			}
		};
		aCenterP.add(memView,BorderLayout.SOUTH);
		memView.setPreferredSize(new Dimension(800,200));

		getContentPane().add(aCenterP, BorderLayout.CENTER);

		//EM 12/10/2008
		JPanel aStatusP = new JPanel();
		aStatusP.setLayout(new BorderLayout());
		getContentPane().add(aStatusP, BorderLayout.SOUTH);
		JPanel aServerP = new JPanel();
		aServerP.setLayout(new FlowLayout());
		aStatusP.add(aServerP, BorderLayout.WEST);
		aServerP.add(new JLabel("JOP app at "));
		aServerP.add(serverIP = new JTextField("localhost"));
		aServerP.add(new JLabel(":"+client._SERVERPORT));
		statusL = new JLabel("Waiting..");
		aStatusP.add(statusL, BorderLayout.CENTER);
		statusL.setBorder(BorderFactory.createBevelBorder(1));

		pack();
		setBounds(10, 10, 800, 600);
		setVisible(true);

		//Event handlers
		aRefreshB.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				updateConsole();
			}
		});
		aSetMarkB.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				setMark();
			}
		});
		aDispFromMarkCB.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				client.dispFromMark = aDispFromMarkCB.isSelected();
				client.threadsTreeRoot.sortRecurse(client.compTimeSpent);
				client.calleesTreeRoot.sortRecurse(client.compTimeSpent);
				//EM 26/10/2008 : use allocsTreeRoot
				client.allocsTreeRoot.sortRecurse(client.compAllocs);
				//EM 28/10/2008
				client.methodsTreeRoot.sortRecurse(client.compTimeSpent);
				
				treeView.repaint();
				memView.repaint();
			}
		});
		aDispJopTimesCB.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				client.dispJopTimes = aDispJopTimesCB.isSelected();
				treeView.repaint();
			}
		});
		//EM 18/10/2008
		viewSelectorCB.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				client.showTree = viewSelectorCB.getSelectedIndex();
				treeView.repaint();
			}
		});
		//EM 21/01/2008
		aForceGcB.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				client.sendForceGC();
			}
		});


		//EM 12/10/2008 : start memory tracker
		Thread aTh = new Thread(new Runnable(){
			public void run() {
				while(true){
					try{
						Thread.sleep(5000);
						trackMem();
					}
					catch(Throwable t){
						//??
					}
				}
			}
		},"MEM TRACKER");
		aTh.start();
	}

	public Console(GraphicsConfiguration arg0) {
		super(arg0);
		// TODO Auto-generated constructor stub
	}

	public Console(String arg0) throws HeadlessException {
		super(arg0);
		// TODO Auto-generated constructor stub
	}

	public Console(String arg0, GraphicsConfiguration arg1) {
		super(arg0, arg1);
		// TODO Auto-generated constructor stub
	}


	//EM 12/10/2008
	void setMark(){
		client.setMark();
		treeView.repaint();
		memView.repaint();
	}

	protected void paintTree(Graphics g) {
		client.displayMethodsNb = 0;
		client.displayAllocsNb = 0;
		treeYPos = treeMaxXPos = 0;
		treeYPos += _LINEHEIGHT;
		Dimension aD = treeView.getSize();
		g.clearRect(0, 0, aD.width, aD.height);

		//EM 26/10/2008
		if(backgroundI != null){
			try{
				double aHI = backgroundI.getHeight(null);
				double aWI = backgroundI.getWidth(null);
				for(int aPosY = 0;aPosY < aD.height + aHI;aPosY+=aHI){
					g.drawImage(backgroundI, 0, aPosY, null);
				}
			}
			catch(Throwable t){
				//??
				t.printStackTrace(System.err);
			}
		}

		//If no tree, display logo
		if(client.threadsTreeRoot == null || client.threadsTreeRoot.subMethodsSorted == null){
			if(logoI != null){
				try{
					double aHI = logoI.getHeight(null);
					double aWI = logoI.getWidth(null);
					double aFact = Math.max(aHI/aD.height,aWI/aD.width);
					aHI/=aFact;
					aWI/=aFact;
					g.drawImage(logoI, (int)((aD.width - aWI)/2), (int)((aD.height - aHI)/2), (int)aWI, (int)aHI, null);
				}
				catch(Throwable t){
					//??
					t.printStackTrace(System.err);
				}
			}
			return;
		}

		//EM 21/10/2008
		switch(client.showTree){
		case _SHOWTREE_METHOD:
		case _SHOWTREE_THREAD:
		case _SHOWTREE_CALLEE:
			paintMethods(g,aD);
			break;
		case _SHOWTREE_ALLOC:
			paintAllocs(g,aD);
			break;
		default:
		}

		Rectangle aSize = treeView.getBounds();
		if(treeYPos + 30 > aSize.height || treeMaxXPos + 30 > aSize.width//
				|| treeYPos - 30 < aSize.height || treeMaxXPos - 30 < aSize.width){
			treeView.setPreferredSize(new Dimension(treeMaxXPos + 30, treeYPos + 30));
			treeView.revalidate();
		}
	}

	//EM 21/10/2008
	//EM 25/10/2008 : use AllocView
	protected void paintAllocs(Graphics g,Dimension aD) {
		paintAllocs(g,aD,0,client.allocsTreeRoot.locationSorted);
	}

	//EM 25/0/2008
	protected void paintAllocs(Graphics g,Dimension aD,int aDepth,TreeSet aAs) {
		if(aAs == null || aAs.size() <= 0){
			return;
		}
		
		Object[] aAllocs = aAs.toArray();
		AllocView aA = ((AllocView)aAllocs[0]);
		long aCountAllocs = aA.count;
		long aCountGCs = aA.countGC;
		if(client.dispFromMark){
			aCountAllocs -= aA.countMark;
			aCountGCs -= aA.countGCMark;
		}
		long aMaxObjs = aCountAllocs - aCountGCs;
		long aMaxAllocs = -1;
		long aMaxGCs = -1;

		String aStr;
		Rectangle2D aR;
		Rectangle aVisibleRect = treeView.getVisibleRect();
		
		//EM 23/10/2008 : colors
		
		for(int t = 0;t < aAllocs.length;t++){
			aA = ((AllocView)aAllocs[t]);
			
			aCountAllocs = aA.count;
			aCountGCs = aA.countGC;
			if(client.dispFromMark){
				aCountAllocs -= aA.countMark;
				aCountGCs -= aA.countGCMark;
			}
			if(aCountAllocs > aMaxAllocs){
				aMaxAllocs = aCountAllocs;
			}
			if(aCountGCs > aMaxGCs){
				aMaxGCs = aCountGCs;
			}
		}

		double aPercent;

		for(int t = 0;t < aAllocs.length;t++){
			aA = ((AllocView)aAllocs[t]);

			//Check displayAllocs size
			if(client.displayAllocsNb >= client.displayAllocs.length){
				AllocView[] aNewAs = new AllocView[client.displayAllocs.length * 2];
				for(int a = 0;a < client.displayAllocs.length;a++){
					aNewAs[a] = client.displayAllocs[a];
				}
				client.displayAllocs = aNewAs;
			}
			
			//Store displayed alloc for interactions
			client.displayAllocs[client.displayAllocsNb] = aA;
			client.displayAllocsNb++;

			int aPosX;
			aPosX = aDepth * 15;

			if(treeYPos > aVisibleRect.y - 15 && treeYPos < aVisibleRect.y + aVisibleRect.height + 15){
				aCountAllocs = aA.count;
				aCountGCs = aA.countGC;
				if(client.dispFromMark){
					aCountAllocs -= aA.countMark;
					aCountGCs -= aA.countGCMark;
				}
				
				aStr = ((aA.locationSorted == null)?"?":((aA.locationSorted.size() <= 0)?"o":(aA.isExpended?"-":"+"))) + " ";
				aR = g.getFontMetrics().getStringBounds(aStr, g);
				g.setColor(Color.BLACK);
				g.drawString(aStr,aPosX, treeYPos);
				aPosX += 20;//aR.getMaxX();

				if(aA.hasCount){
					aPercent = (aMaxObjs <= 0) ? 0 : 100.0 * (double)(aCountAllocs - aCountGCs) / (double)aMaxObjs;
					aPercent = (aPercent <= 0) ? 0 : (aPercent >= 100) ? 100 : aPercent;//Due to mak pos	
					aStr = " obj" + (aCountAllocs - aCountGCs) + " ";
					aR = g.getFontMetrics().getStringBounds(aStr, g);
					g.setColor(Colors.browns[(int)(2.25 * (100 - aPercent))]);
					g.fillRect(aPosX, treeYPos - 12, (int)aR.getWidth(), (int)aR.getHeight());
					g.setColor(Color.BLACK);
					g.drawString(aStr,aPosX, treeYPos);
					aPosX += aR.getMaxX();

					aPercent = (aMaxAllocs <= 0) ? 0 : 100.0 * (double)aCountAllocs / (double)aMaxAllocs;
					aStr = " alloc" + aCountAllocs + " ";
					aR = g.getFontMetrics().getStringBounds(aStr, g);
					g.setColor(Colors.magentas[(int)(2.25 * (100 - aPercent))]);
					g.fillRect(aPosX, treeYPos - 12, (int)aR.getWidth(), (int)aR.getHeight());
					g.setColor(Color.BLACK);
					g.drawString(aStr,aPosX, treeYPos);
					aPosX += aR.getMaxX();

					//aPercent = (aMaxGCs <= 0) ? 0 : 100.0 * (double)aCountGCs / (double)aMaxGCs;
					aPercent = (aCountAllocs <= 0) ? 0 : 100.0 * (double)aCountGCs / (double)aCountAllocs;
					aPercent = (aPercent <= 0) ? 0 : (aPercent >= 100) ? 100 : aPercent;//Due to mak pos	
					aStr = " gc" + aCountGCs + " ";
					aR = g.getFontMetrics().getStringBounds(aStr, g);
					g.setColor(Colors.oranges[(int)(2.25 * (aPercent))]);
					g.fillRect(aPosX, treeYPos - 12, (int)aR.getWidth(), (int)aR.getHeight());
					g.setColor(Color.BLACK);
					g.drawString(aStr,aPosX, treeYPos);
					aPosX += aR.getMaxX();

					aPercent = (aCountAllocs <= 0) ? 0 : 100.0 * (double)aCountGCs / (double)aCountAllocs;
					aPercent = (aPercent <= 0) ? 0 : (aPercent >= 100) ? 100 : aPercent;//Due to mak pos	
					aStr = " " + nf.format(aPercent) + "% ";
					aR = g.getFontMetrics().getStringBounds(aStr, g);
					g.setColor(Colors.cyans[(int)(2.25 * aPercent)]);
					g.fillRect(aPosX, treeYPos - 12, (int)aR.getWidth(), (int)aR.getHeight());
					g.setColor(Color.BLACK);
					g.drawString(aStr,aPosX, treeYPos);
					aPosX += aR.getMaxX();
				}
				
				//EM 29/10/2008 : different font for each part of the method descr
				AllocView aM = aA;
				if(aM.methodPath != null){
					g.setFont(smallFont);
					aStr = " " + aM.methodPath;
					aR = g.getFontMetrics().getStringBounds(aStr, g);
					g.drawString(aStr,aPosX, treeYPos);
					g.setColor(Color.BLACK);
					aPosX += aR.getMaxX();

					g.setFont(boldFont);
					aStr = " " + aM.methodName;
					aR = g.getFontMetrics().getStringBounds(aStr, g);
					g.drawString(aStr,aPosX, treeYPos);
					g.setColor(Color.BLACK);
					aPosX += aR.getMaxX();

					g.setFont(smallFont);
					aStr = " " + aM.methodAttr;
					aR = g.getFontMetrics().getStringBounds(aStr, g);
					g.drawString(aStr,aPosX, treeYPos);
					g.setColor(Color.BLACK);
					aPosX += aR.getMaxX();

					g.setFont(smallFont);
					aStr = " " + aM.methodRet;
					aR = g.getFontMetrics().getStringBounds(aStr, g);
					g.drawString(aStr,aPosX, treeYPos);
					g.setColor(Color.BLACK);
					aPosX += aR.getMaxX();
					g.setFont(stdFont);
				}
				else{
					g.setFont(boldFont);
					aStr = " " + (String)aA.name + " ";
					aR = g.getFontMetrics().getStringBounds(aStr, g);
					//g.setColor(Colors.cyans[(int)(2.25 * (100 - aPercent))]);
					//g.fillRect(aPosX, treeYPos - 12, (int)aR.getWidth(), (int)aR.getHeight());
					g.setColor(Color.BLACK);
					g.drawString(aStr,aPosX, treeYPos);
					aPosX += aR.getMaxX();
					g.setFont(stdFont);
				}
			}
			if(aPosX > treeMaxXPos){
				treeMaxXPos = aPosX;
			}
			treeYPos += _LINEHEIGHT;
			
			if(aA.locationSorted != null && aA.isExpended){
				paintAllocs(g,aD,aDepth + 1,aA.locationSorted) ;
			}
		}
	}

	//EM 21/10/2008
	protected void paintMethods(Graphics g,Dimension aD) {
		//Display tree
		Object[] aThs = null;
		long aMaxCount = -1;
		long aMaxTimeSpent = -1;
		switch(client.showTree){
		case _SHOWTREE_THREAD:
			aThs = client.threadsTreeRoot.subMethodsSorted.toArray();
			break;
		case _SHOWTREE_METHOD:
			aThs = client.methodsTreeRoot.subMethodsSorted.toArray();
			break;
		case _SHOWTREE_CALLEE:
		default:{
			aThs = client.calleesTreeRoot.subMethodsSorted.toArray();
			for(int m = 0;m < aThs.length;m++){
				if(aMaxCount < ((MethodView)aThs[m]).count){
					aMaxCount = ((MethodView)aThs[m]).count;
				}
			}
			for(int m = 0;m < aThs.length && aMaxTimeSpent <= 0;m++){
				if(!((MethodView)aThs[m]).isWait()){
					aMaxTimeSpent = ((MethodView)aThs[m]).timeSpent;
				}
			}
			break;
		}
		}

		for(int t = 0;t < aThs.length;t++){
			MethodView aM = (MethodView)aThs[t];
			long aTimeSpent = client.dispFromMark ? aM.timeSpent - aM.timeSpentMark : aM.timeSpent;
			long aTimeSubSpent = client.dispFromMark ? aM.timeSubSpent - aM.timeSubSpentMark : aM.timeSubSpent;
			long aTimeInSpentSubMax = client.dispFromMark ? aM.timeInSpentSubMax - aM.timeInSpentSubMaxMark : aM.timeInSpentSubMax;
			long aCountSubMax = client.dispFromMark ? aM.countSubMax - aM.countSubMaxMark : aM.countSubMax;
			long aTotTime = (aTimeSpent > aTimeSubSpent)?aTimeSpent:aTimeSubSpent;
			paintMethod(g,aM,0,(aMaxTimeSpent > 0)?aMaxTimeSpent:aTotTime,aTimeInSpentSubMax,(aMaxCount > 0)?aMaxCount:aCountSubMax);
		}
	}

	protected void paintMethod(Graphics g,MethodView aM,int aDepth, long aTotTime, long aMaxInTime,long aMaxCount) {
		//EM 19/10/2008
		Rectangle aVisibleRect = treeView.getVisibleRect();

		//EM 29/10/2008
		g.setFont(stdFont);
		
		//Check displayMethods size
		if(client.displayMethodsNb >= client.displayMethods.length){
			MethodView[] aNewDM = new MethodView[client.displayMethods.length * 2];
			for(int m = 0;m < client.displayMethods.length;m++){
				aNewDM[m] = client.displayMethods[m];
			}
			client.displayMethods = aNewDM;
		}
		//Store displayed method for interactions
		client.displayMethods[client.displayMethodsNb] = aM;
		client.displayMethodsNb++;

		//Paint method
		long aCount = client.dispFromMark ? aM.count - aM.countMark : aM.count;
		long aCountSubMax = client.dispFromMark ? aM.countSubMax - aM.countSubMaxMark : aM.countSubMax;
		long aTimeSpent = client.dispFromMark ? aM.timeSpent - aM.timeSpentMark : aM.timeSpent;
		long aTimeInSpent = client.dispFromMark ? aM.timeInSpent - aM.timeInSpentMark : aM.timeInSpent;
		long aTimeSubSpent = client.dispFromMark ? aM.timeSubSpent - aM.timeSubSpentMark : aM.timeSubSpent;
		long aTimeJopSpent = client.dispFromMark ? aM.timeJopSpent - aM.timeJopSpentMark : aM.timeJopSpent;
		long aTimeJopSpentTot = client.dispFromMark ? aM.timeJopSpentTot - aM.timeJopSpentTotMark : aM.timeJopSpentTot;
		long aTimeInSpentSubMax = client.dispFromMark ? aM.timeInSpentSubMax - aM.timeInSpentSubMaxMark : aM.timeInSpentSubMax;

		//EM 29/10/2008 : percents can be out of 0-100 because of mark
		//long aLocalTime = (aTimeSpent > aTimeSubSpent)?aTimeSpent:aTimeSubSpent;
		long aNewTotTime = (aTotTime > aTimeSpent) ? aTotTime : aTimeSpent;
		double aPercent = (aNewTotTime <= 0) ? 0 : (aTimeSpent  * 100.0) / aNewTotTime;
		aPercent = (aPercent <= 0) ? 0 : ((aPercent >= 100)? 100 : aPercent );
		long aNewMaxInTime = (aMaxInTime > aTimeInSpentSubMax) ? aMaxInTime : aTimeInSpentSubMax;
		double aInPercent = (aNewMaxInTime <= 0) ? 0 : (aTimeInSpent  * 100.0) / aNewMaxInTime;
		aInPercent = (aInPercent <= 0) ? 0 : ((aInPercent >= 100)? 100 : aInPercent );
		long aNewMaxCount = (aMaxCount > aCountSubMax) ? aMaxCount : aCountSubMax;
		double aCountPercent = (aCount <= 0) ? 0 : (aCount  * 100.0) / aNewMaxCount;
		aCountPercent = (aCountPercent <= 0) ? 0 : ((aCountPercent >= 100)? 100 : aCountPercent );

		int aPosX;
		aPosX = aDepth * 15;

		//EM 19/10/2008 : paint only if visible
		if(treeYPos > aVisibleRect.y - 15 && treeYPos < aVisibleRect.y + aVisibleRect.height + 15){
			String aStr;
			Rectangle2D aR;


			aStr = ((aM.subMethodsSorted == null)?"?":((aM.subMethodsSorted.size() <= 0)?"o":(aM.isExpended?"-":"+"))) + " ";
			aR = g.getFontMetrics().getStringBounds(aStr, g);
			g.setColor(Color.BLACK);
			g.drawString(aStr,aPosX, treeYPos);
			aPosX += 20;//aR.getMaxX();

			if(client.showTree == _SHOWTREE_METHOD || client.showTree == _SHOWTREE_THREAD){
				aStr = " " + nf.format(aPercent) + "% ";
				aR = g.getFontMetrics().getStringBounds(aStr, g);
				g.setColor(Colors.cyans[(int)(2.25 * (100 - aPercent))]);
				g.fillRect(aPosX, treeYPos - 12, (int)aR.getWidth(), (int)aR.getHeight());
				g.setColor(Color.BLACK);
				g.drawString(aStr,aPosX, treeYPos);
				aPosX += aR.getMaxX();

				aStr = " tot" + formatTime(aTimeSpent) + " ";
				aR = g.getFontMetrics().getStringBounds(aStr, g);
				g.setColor(Colors.reds[(int)(2.25 * (100 - aPercent))]);
				g.fillRect(aPosX, treeYPos - 12, (int)aR.getWidth(), (int)aR.getHeight());
				g.setColor(Color.BLACK);
				g.drawString(aStr,aPosX, treeYPos);
				aPosX += aR.getMaxX();

				//EM 18/10/2008
				if(client.dispJopTimes){
					//EM 11/10/2008
					String aOverload = (aTimeSpent == 0)? "-" : nf.format((double)aTimeJopSpentTot / (double)aTimeSpent) + "x";
					aStr = " (jop" + formatTime(aTimeJopSpentTot) + " over"+aOverload+") " + " ";
					aR = g.getFontMetrics().getStringBounds(aStr, g);
					g.setColor(Color.BLACK);
					g.drawString(aStr,aPosX, treeYPos);
					aPosX += aR.getMaxX();
				}

				aStr = aTimeSpent <= 0 ? "" : " in" + formatTime(aTimeInSpent) + " ";
				aR = g.getFontMetrics().getStringBounds(aStr, g);
				g.setColor(Colors.oranges[(int)(2.25 * (100 - aInPercent))]);
				g.fillRect(aPosX, treeYPos - 12, (int)aR.getWidth(), (int)aR.getHeight());
				g.setColor(Color.BLACK);
				g.drawString(aStr,aPosX, treeYPos);
				aPosX += aR.getMaxX();

				//EM 18/10/2008
				if(client.dispJopTimes){
					//EM 11/10/2008
					String aOverload = ((aTimeSpent - aTimeSubSpent) == 0)? "-" : nf.format((double)aTimeJopSpent / (double)(aTimeSpent - aTimeSubSpent)) + "x";
					aStr = " (jop" + formatTime(aTimeJopSpent) + " over"+aOverload+") " + " ";
					aR = g.getFontMetrics().getStringBounds(aStr, g);
					g.setColor(Color.BLACK);
					g.drawString(aStr,aPosX, treeYPos);
					aPosX += aR.getMaxX();
				}
			}
			else if(client.showTree == _SHOWTREE_CALLEE){
				aStr = " cumul" + formatTime(aTimeSpent) + " ";
				aR = g.getFontMetrics().getStringBounds(aStr, g);
				g.setColor(Colors.magentas[(int)(2.25 * (100 - aPercent))]);
				g.fillRect(aPosX, treeYPos - 12, (int)aR.getWidth(), (int)aR.getHeight());
				g.setColor(Color.BLACK);
				g.drawString(aStr,aPosX, treeYPos);
				aPosX += aR.getMaxX();

				//EM 18/10/2008
				if(client.dispJopTimes){
					//EM 11/10/2008
					String aOverload = (aTimeSpent == 0)? "-" : nf.format((double)aTimeJopSpent / aTimeSpent) + "x";
					aStr = " (jop" + formatTime(aTimeJopSpent) + " over"+aOverload+") " + " ";
					aR = g.getFontMetrics().getStringBounds(aStr, g);
					g.setColor(Color.BLACK);
					g.drawString(aStr,aPosX, treeYPos);
					aPosX += aR.getMaxX();
				}
			}

			aStr = (aCount <= 0) ? "" : " " + aCount + "x ";
			aR = g.getFontMetrics().getStringBounds(aStr, g);
			g.setColor(Colors.greens[(int)(2.25 * (100 - aCountPercent))]);
			g.fillRect(aPosX, treeYPos - 12, (int)aR.getWidth(), (int)aR.getHeight());
			g.setColor(Color.BLACK);
			g.drawString(aStr,aPosX, treeYPos);
			aPosX += aR.getMaxX();

			//EM 29/10/2008 : different font for each part of the method descr
			if(aM.methodPath != null){
				g.setFont(smallFont);
				aStr = " " + aM.methodPath;
				aR = g.getFontMetrics().getStringBounds(aStr, g);
				g.drawString(aStr,aPosX, treeYPos);
				g.setColor(Color.BLACK);
				aPosX += aR.getMaxX();

				g.setFont(boldFont);
				aStr = " " + aM.methodName;
				aR = g.getFontMetrics().getStringBounds(aStr, g);
				g.drawString(aStr,aPosX, treeYPos);
				g.setColor(Color.BLACK);
				aPosX += aR.getMaxX();

				g.setFont(smallFont);
				aStr = " " + aM.methodAttr;
				aR = g.getFontMetrics().getStringBounds(aStr, g);
				g.drawString(aStr,aPosX, treeYPos);
				g.setColor(Color.BLACK);
				aPosX += aR.getMaxX();

				g.setFont(smallFont);
				aStr = " " + aM.methodRet;
				aR = g.getFontMetrics().getStringBounds(aStr, g);
				g.drawString(aStr,aPosX, treeYPos);
				g.setColor(Color.BLACK);
				aPosX += aR.getMaxX();
			}
			else{
				g.setFont(boldFont);
				aStr = " " + aM.methodSign;
				aR = g.getFontMetrics().getStringBounds(aStr, g);
				if(aM.methodSign.indexOf("(JOPed)") >= 0){
					g.setColor(Color.white);
					g.fillRect(aPosX, treeYPos - 12, (int)aR.getWidth(), (int)aR.getHeight());
					g.setColor(Color.black);
				}
				g.drawString(aStr,aPosX, treeYPos);
				g.setColor(Color.BLACK);
				aPosX += aR.getMaxX();
				g.setFont(stdFont);
			}
		}

		if(aPosX > treeMaxXPos){
			treeMaxXPos = aPosX;
		}
		treeYPos += _LINEHEIGHT;
		if(!aM.isExpended){
			//Don't paint lower
			return;
		}

		Object[] aMs = aM.subMethodsSorted.toArray();
		for(int m = 0;m < aMs.length;m++){
			paintMethod(g,(MethodView)aMs[m],aDepth+1,aNewTotTime,aNewMaxInTime,aNewMaxCount);
		}
	}

	//EM 12/10/2008
	protected void paintMem(Graphics g) {
		try{
			Dimension aD = memView.getSize();
			g.clearRect(0, 0, aD.width, aD.height);

			if(client.memTot.size() - (client.dispFromMark ? client.markTimePos : 0) < 2){
				return;
			}

			//Search max val
			long aMaxTot = 0;
			long aMaxObjs = 0;
			for(int i = (client.dispFromMark ? client.markTimePos : 0);i < client.memTot.size();i++){
				long aTot = (Long)client.memTot.elementAt(i);
				if(aMaxTot < aTot){
					aMaxTot = aTot;
				}
				long aObjs = (Long)client.allocsTot.elementAt(i) - (Long)client.gcTot.elementAt(i);
				if(aMaxObjs < aObjs){
					aMaxObjs = aObjs;
				}
			}
			double aYFactor = (double)aD.height / (double)aMaxTot;
			double aYObjsFactor = (double)aD.height / (double)aMaxObjs;

			//Paint
			double aStep = (double)aD.width / (double)(client.memTot.size() - (client.dispFromMark ? client.markTimePos : 0) - 1);

			if(client.updateTimePos > 0){
				//EM 18/10/2008
				g.setColor(Color.white);
				g.fillRect(0, 0, (int)((client.updateTimePos - (client.dispFromMark ? client.markTimePos : 0)) *aStep), aD.height);

				g.setColor(Color.black);
				g.drawLine((int)((client.updateTimePos - (client.dispFromMark ? client.markTimePos : 0)) *aStep), 0, (int)((client.updateTimePos - (client.dispFromMark ? client.markTimePos : 0)) * aStep), aD.height);
				g.drawString("Update", (int)((client.updateTimePos - (client.dispFromMark ? client.markTimePos : 0)) *aStep)+2, aD.height-5);
			}
			if(client.markTimePos > 0){
				g.setColor(Color.red);
				g.drawLine((int)((!client.dispFromMark ? client.markTimePos : 0) *aStep), 0, (int)((!client.dispFromMark ? client.markTimePos : 0)*aStep), aD.height);
				g.drawString("Mark", (int)((!client.dispFromMark ? client.markTimePos : 0) *aStep)+2, aD.height-_LINEHEIGHT - 5);
			}

			//Curves
			long aTotLast = -1;
			long aFreeLast = -1;
			long aAllocsLast = -1;
			for(int m = (client.dispFromMark ? client.markTimePos : 0);m < client.memTot.size();m++){
				long aTot = (Long)client.memTot.elementAt(m);
				long aFree = (Long)client.memFree.elementAt(m);
				long aAllocs = (Long)client.allocsTot.elementAt(m) - (Long)client.gcTot.elementAt(m);
				if(aTotLast > 0){
					int i = m - (client.dispFromMark ? client.markTimePos : 0);
					g.setColor(Color.green);
					g.drawLine((int)((i - 1)*aStep), aD.height - (int)(aTotLast * aYFactor), (int)(i*aStep), aD.height - (int)(aTot * aYFactor));
					g.setColor(Color.blue);
					g.drawLine((int)((i - 1)*aStep), aD.height - (int)((aTotLast - aFreeLast) * aYFactor), (int)(i*aStep), aD.height - (int)((aTot - aFree) * aYFactor));
					g.setColor(Colors.browns[0]);
					g.drawLine((int)((i - 1)*aStep), aD.height - (int)((aAllocsLast) * aYObjsFactor), (int)(i*aStep), aD.height - (int)((aAllocs) * aYObjsFactor));
				}
				aTotLast = aTot;
				aFreeLast = aFree;
				aAllocsLast = aAllocs;
			}

			g.setColor(Color.green);
			g.drawString("MemMax=" + (aMaxTot / (1024*1024)) + "M", aD.width - 150, aD.height - 60);
			g.setColor(Color.blue);
			g.drawString("MemUsed=" + (((Long)client.memTot.elementAt(client.memTot.size() - 1) - (Long)client.memFree.elementAt(client.memFree.size() - 1)) / (1024*1024)) + "M", aD.width - 150, aD.height - 45);
			g.setColor(new Color(0x5c1d00));
			g.drawString("Objects=" + (((Long)client.allocsTot.elementAt(client.allocsTot.size() - 1) - (Long)client.gcTot.elementAt(client.gcTot.size() - 1))), aD.width - 150, aD.height - 30);
		}
		catch(Throwable t){
			//Something wrong ?? Perhaps a data changing during paint
			t.printStackTrace(System.err);
		}
	}

	//EM 12/10/2008
	void trackMem(){
		client.trackMem();
		memView.repaint();
	}
	
	void updateConsole(){
		client.updateTree();
		treeView.repaint();
	}

	public String formatTime(long aT){
		long aMStot = aT / 1000000L;
		long aStot = aMStot / 1000;
		long aMtot = aStot/60;
		long aHtot = aMtot/60;
		if(aHtot > 0){
			return aHtot + "h" + (aMtot % 60) + "m";
		}
		if(aMtot > 0){
			return (aMtot % 60) + "m" + (aStot % 60) + "s";
		}
		return (aStot % 60) + "s" + (aMStot % 1000) + "ms";
	}

	public static void main(String[] args) {
		Console aConsole = new Console();
	}

}
