package em.jop;

import java.io.BufferedOutputStream;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import em.jop.profiler.Alloc;
import em.jop.profiler.JOPThread;
import em.jop.profiler.Method;
import em.jop.profiler.Server;
import em.jop.profiler.ThreadStack;


public final class Profiler implements Runnable {
	static final boolean _DEBUG_ALL = false;
	static final boolean _DEBUG_ALLOC = false;
	static final boolean _DEBUG_WAIT = false;
	static final boolean _DEBUG_UNWIND = true;
	static final boolean _DEBUG_END_CHECK = false;
//	static long _startTime;

	static Server server = null;

	//EM 11/10/2008 : sync thread map updates
	public static Map<Long,ThreadStack> threadMap = Collections.synchronizedMap(new HashMap<Long,ThreadStack>(1024));
	//EM 11/10/2008
	static int countDumped = 0;
	static Object lockProfDump = new Object();

	//EM 12/10/2008
	static Map<String,Alloc> allocMap = Collections.synchronizedMap(new HashMap<String,Alloc>(1024));
	//EM 21/10/2008
	static public long countAllocs = 0;
	static public long countGC = 0;

	public static void initProfiler() {
		System.err.println("EMProfiler: starting");
		Runtime.getRuntime().addShutdownHook(new Thread(new Profiler()));

//		if (Controller._timeResoltion == Controller.TimeResolution.ns) {
//		_startTime = System.nanoTime();
//		} else {
//		_startTime = System.currentTimeMillis();
//		}

		//Init JOP server
		try{
			server = new Server();
		}
		catch(Throwable t){
			t.printStackTrace(System.err);
			System.exit(-1);
		}
	}


	//EM 28/10/2008 : directly use the method sign
	//public static void start(String className, String methodName) {
	public static void start(String aMethodSign) {
		long aInTime = System.nanoTime();
		//EM 11/10/2008 : lock when dumping
		/*synchronized(lockProfDump)*/{
			//EM 28/10/2008 : directly use the method sign
			if(_DEBUG_ALL) System.out.println("START " + aMethodSign);//className + "." + methodName);
			ThreadStack aThStack = null;
			Method aMethod = null;
			try{
				//EM 28/10/2008 : directly use the method sign
				//String aMethodSign = className + "." + methodName;

				//Find thread stack
				//EM 16/10/2008
				Thread aTh = Thread.currentThread();
				if(aTh instanceof JOPThread){
				aThStack = ((JOPThread)aTh).threadStack;//threadMap.get(aThId);
//				System.out.println("JOPThread for "+ aMethodSign);
				}
				else{
				aThStack = threadMap.get(aTh.getId());
//				System.out.println("STD Thread for "+ aMethodSign);
				}
				//Find already existing method from already existing parent
				//Error, this is not so direct
				Method aMethodParent = null;
				try{
					//Is it possible to find the method a parent already in the stack ?
					aMethodParent = aThStack.stack[aThStack.stackDepth - 1];
					aMethod = aMethodParent.subMethods.get(aMethodSign);
				}
				catch(Throwable t){
					//Use catch to avoid a systematic tests : in most cases, the method and the method's parent are found
					if(aThStack == null){
						//No already existing stack, create new one
						threadMap.put(aTh.getId(), aThStack = new ThreadStack(Thread.currentThread().getId(),aTh.getName()));
					}
				}
				if(aMethod == null){//Don't know how this test could be avoided...??
					aMethod = new Method(aMethodSign);
					//Add method to parent
					try{
						aMethodParent.subMethods.put(aMethodSign, aMethod);
					}
					catch(Throwable t){
					}
				}
				//Add method to stack
				try{
					aThStack.stack[aThStack.stackDepth] = aMethod; 
					aThStack.stackDepth++;
				}
				catch(Throwable t){
					//Enlarge stack. Using catch avoid a systematic test "if(aThStack.stackPos >= aThStack.stack.length)", not a lot of cases
					if(aThStack != null && aMethod != null //
							&& aThStack.stackDepth >= aThStack.stack.length){
						Method[] aNewStack = new Method[aThStack.stack.length * 2];
						for(int i = 0;i < aThStack.stack.length;i++){
							aNewStack[i] = aThStack.stack[i];
						}
						aThStack.stack = aNewStack;
						aThStack.stack[aThStack.stackDepth] = aMethod;
						aThStack.stackDepth++;
					}
				}

				//EM 06/10/2008 : Set time
				//Count
				aMethod.count++;
				aMethod.timeStart = aInTime;
				aMethod.timeJopSpent += System.nanoTime() - aInTime;
			}
			catch(Throwable t){
				//Something wrong ???
				t.printStackTrace(System.err);
			}
		}
	}

	//EM 28/10/2008 : Even, no need to have paramters here, only for debug
	//EM 28/10/2008 : directly use the method sign
	//public static void end(String className, String methodName) {
	//public static void end(String aMethodSign) {
	public static void end() {
		long aInTime = System.nanoTime();
		//EM 11/10/2008 : lock when dumping
		/*synchronized(lockProfDump)*/{
			//EM 28/10/2008 : Even, no need to have paramters here, only for debug
			//if(_DEBUG_ALL) System.out.println("END   " + aMethodSign);//className + "." + methodName);
			try{
				//Find thread stack
				//EM 16/10/2008
				Thread aTh = Thread.currentThread();
				ThreadStack aThStack = null;
				if(aTh instanceof JOPThread){
				aThStack = ((JOPThread)aTh).threadStack;//threadMap.get(aThId);
				}
				else{
				aThStack = threadMap.get(aTh.getId());
				}
				//Update method, and remove from stack
				aThStack.stackDepth--;
				Method aMethod = aThStack.stack[aThStack.stackDepth]; 

				//EM 28/10/2008 : Even, no need to have paramters here, only for debug
//				if(_DEBUG_END_CHECK){
//					//EM 28/10/2008 : directly use the method sign
//					//String aMethodSign = className + "." +methodName;
//					if(!aMethod.methodSign.equalsIgnoreCase(aMethodSign)){
//						System.err.println("BAD END : " + aMethod.methodSign + " / " + aMethodSign);
//					}
//				}

				//EM 06/10/2008
				long aOutTime = System.nanoTime();
				aMethod.timeSpent += aOutTime - aMethod.timeStart;
				aMethod.timeJopSpent += aOutTime - aInTime;
			}
			catch(Throwable t){
				//Something wrong ??
				t.printStackTrace(System.err);
			}
		}
	}

	//EM 23/10/2008 : test caller to keep only first init
	public static void alloc(Object aCaller,String classNameObj,String className) {
		long aInTime = System.nanoTime();

		try{
			if(_DEBUG_ALL || _DEBUG_ALLOC) System.out.println("ALLOC [" 
					+ aCaller.hashCode() + "] " 
					+ aCaller.getClass().getName() + ":" + classNameObj);

			//Find thread stack
			//EM 16/10/2008
			Thread aTh = Thread.currentThread();
			ThreadStack aThStack = null;
			if(aTh instanceof JOPThread){
			aThStack = ((JOPThread)aTh).threadStack;//threadMap.get(aThId);
			}
			else{
			aThStack = threadMap.get(aTh.getId());
			}

			//EM 25/10/2008 : test the caller and the last method to keep only the first init (other init method of the same class with different args) 
			Method aMethod = aThStack.stack[aThStack.stackDepth - 2]; 
			if(!aCaller.getClass().getName().equals(classNameObj) //
					//EM 28/10/2008 : use startsWith() and not equals() because signature is included
					|| aMethod.methodSign.startsWith(className + ".<init>")){//Need to have it without replacements..
				aMethod.timeJopSpent += System.nanoTime() - aInTime;
				return;
			}

			//Alloc count
			countAllocs++;

			Alloc aAlloc = allocMap.get(classNameObj);
			try{
				aAlloc.counts++;
			}
			catch(Throwable t2){
				aAlloc = new Alloc();
				aAlloc.counts++;
				allocMap.put(classNameObj, aAlloc);
			}


			if(_DEBUG_ALL || _DEBUG_ALLOC){
				System.out.println("ALLOC in : " + aMethod.methodSign + " : " + classNameObj);
			}

			//Location
			Alloc aAllocLoc = aAlloc.location.get(aMethod.methodSign);
			try{
				aAllocLoc.counts++;
			}
			catch(Throwable t2){
				aAllocLoc = new Alloc();
				aAllocLoc.counts++;
				aAlloc.location.put(aMethod.methodSign, aAllocLoc);
			}
			//EM 26/10/2008
			aAlloc.locationTrack.put(new Integer(aCaller.hashCode()), aAllocLoc);

			//EM 06/10/2008
			aMethod.timeJopSpent += System.nanoTime() - aInTime;
		}
		catch(Throwable t){
			//Something wrong ??
			t.printStackTrace(System.err);
		}

	}

	//EM 20/10/2008
	//EM 23/10/2008 : test caller to keep only first init
	public static void gc(Object aCaller,String className) {
		if(_DEBUG_ALL || _DEBUG_ALLOC) System.out.println("GC " + aCaller.getClass().getName() + ":" + className);
		//EM 23/10/2008 : test caller to keep only first init
		if(!aCaller.getClass().getName().equals(className)){
			return;
		}
		countGC++;

		try{
			//Find thread stack
			//EM 16/10/2008
			Thread aTh = Thread.currentThread();
			ThreadStack aThStack = null;
			if(aTh instanceof JOPThread){
			aThStack = ((JOPThread)aTh).threadStack;//threadMap.get(aThId);
			}
			else{
			aThStack = threadMap.get(aTh.getId());
			}

			//Alloc count
			Alloc aAlloc = allocMap.get(className);
			
			//EM 26/10/2008 : location
			Integer aKey = new Integer(aCaller.hashCode());
			Alloc aAllocLoc = aAlloc.locationTrack.get(aKey);
			if(aAllocLoc != null){
				aAllocLoc.gcs++;
				aAlloc.locationTrack.remove(aKey);
			}
			else{
				//Due to hascode limitations, some GCed instances can be not located
				aAllocLoc = aAlloc.location.get("NOT LOCATED GCed INSTANCES");
				if(aAllocLoc == null){
					aAllocLoc = new Alloc();
					aAlloc.location.put("NOT LOCATED GCed INSTANCES", aAllocLoc);
				}
				aAllocLoc.gcs++;
			}

			try{
				aAlloc.gcs++;
			}
			catch(Throwable t2){
				//Should never occur ??
			}



//			Method aMethod = aThStack.stack[aThStack.stackDepth--]; 

//			if(_DEBUG_ALL || _DEBUG_ALLOC){
//			System.out.println("ALLOC in : " + aMethod.methodSign + " : " + className + "(" + aCaller + ")");
//			}


//			Alloc aAllocLoc = aAlloc.location.get(aMethod.methodSign);
//			try{
//			aAllocLoc.counts++;
//			}
//			catch(Throwable t2){
//			aAllocLoc = new Alloc();
//			aAllocLoc.counts++;
//			aAlloc.location.put(aMethod.methodSign, aAllocLoc);
//			}

			//EM 06/10/2008
//			aMethod.timeJopSpent += System.nanoTime() - aInTime;
		}
		catch(Throwable t){
			//Something wrong ??
			t.printStackTrace(System.err);
		}
//		Long[] aCounts = (Long[])allocMap.get(className);
//		try{
//		aCounts[1]++;
//		}
//		catch(Throwable t){
//		//Strange ??
//		aCounts = new Long[2];
//		aCounts[0] = new Long(0);
//		aCounts[1] = new Long(1);
//		allocMap.put(className, aCounts);
//		}
	}

	public static void beginWait(String className, String methodName) {
		if(_DEBUG_ALL || _DEBUG_WAIT) System.out.println("BEG WAIT " + className);
		//EM 28/10/2008 : directly use the method sign
		//start("", "WAITING");
		start("WAITING");
	}

	public static void endWait(String className, String methodName) {
		if(_DEBUG_ALL || _DEBUG_WAIT) System.out.println("END WAIT " + className);
		//EM 28/10/2008 : Even, no need to have paramters here, only for debug
		//EM 28/10/2008 : directly use the method sign
		//end("", "WAITING");
		//end("WAITING");
		end();
	}

	public static void unwind(String className, String methodName, String exception) {
		//EM 11/10/2008 : lock when dumping
		/*synchronized(lockProfDump)*/{
			if(_DEBUG_ALL || _DEBUG_UNWIND) System.out.println("UNWIND " + className + "." + methodName + ":" + exception);
			try{
				//Find thread stack
				//EM 16/10/2008
//				Thread aTh = Thread.currentThread();
//				Long aThId = aTh.getId();
//				ThreadStack aThStack = threadMap.get(aThId);
				Thread aTh = Thread.currentThread();
				ThreadStack aThStack = null;
				if(aTh instanceof JOPThread){
				aThStack = ((JOPThread)aTh).threadStack;//threadMap.get(aThId);
				}
				else{
				aThStack = threadMap.get(aTh.getId());
				}
				//Update method, and remove from stack
				//Search method in the stack
				String aMethodSign = className + "." +methodName;
				long aTime = System.nanoTime();
				for(int i = aThStack.stackDepth - 1;i >= 0;i--){
					Method aMethod = aThStack.stack[i]; 
					if(aThStack.stack[i].methodSign.equalsIgnoreCase(aMethodSign)){
						//Found !
						aThStack.stackDepth = i+1;
						break;
					}
					//update all intermediate methods times
					aMethod.timeSpent += aTime - aMethod.timeStart;
				}
			}
			catch(Throwable t){
				//Something wrong ??
				t.printStackTrace(System.err);
			}
		}
	}

	public static void dump(BufferedOutputStream out) throws Exception {
		//EM 11/10/2008 : lock when dumping
		/*synchronized(lockProfDump)*/{
			//For each threads
			Object[] ths = (Object[])threadMap.values().toArray();
			//System.out.println("---Update view for "+ths.length+" threads---");
			//EM 11/10/008
			countDumped = 0;

			for(int i = 0;i < ths.length;i++){
				ThreadStack aTh = (ThreadStack)ths[i];
				out.write(("TH\t"+aTh.id+"\t"+aTh.threadName+"\n").getBytes("iso-8859-1"));
				String aThSign = "[TH" + aTh.id + "]"+ aTh.threadName;
				dump(out,aTh.stack[0]);
			}
			System.out.println("dumped " + countDumped + " calls");
			out.write("END\n".getBytes("iso-8859-1"));
			out.flush();
		}
	}

	static void dump(BufferedOutputStream out,Method aProfM)  throws Exception {
		try{
			out.write(("SUB\t"+aProfM.methodSign+"\t"+aProfM.timeSpent+"\t"+aProfM.count+"\t"+aProfM.timeJopSpent+"\n").getBytes("iso-8859-1"));
			//EM 11/10/008
			if((++countDumped % 1000) == 0){
				//System.out.println("dump " + countDumped + " calls..");
			}

			//System.out.println("---" + aNewM.methodSign);
			Object[] aSubs = (Object[])aProfM.subMethods.values().toArray();
			for(int m = 0;m < aSubs.length;m++){
				Method aProfSub = (Method)aSubs[m];
				dump(out,aProfSub);
			}
			out.write("UP\n".getBytes("iso-8859-1"));
		}
		catch(Throwable t){
			//??
			t.printStackTrace(System.err);
		}
	}

	//EM 21/10/2008
	public static void dumpAllocs(BufferedOutputStream out) throws Exception {
		//EM 11/10/2008 : lock when dumping
		/*synchronized(lockProfDump)*/{
			//For each threads
			Object[] aNames = (Object[])allocMap.keySet().toArray();
			//System.out.println("---Update view for "+ths.length+" threads---");
			//EM 11/10/008
			countDumped = 0;

			for(int i = 0;i < aNames.length;i++){
				String aName = (String)aNames[i];
				Alloc aAlloc = allocMap.get(aName);
//				Long[] aCounts = (Long[])allocMap.get(aName);
				out.write(("ALLOC\t" + aName+"\t"+aAlloc.counts+"\t"+aAlloc.gcs+"\n").getBytes("iso-8859-1"));
				Object[] aLocNames = aAlloc.location.keySet().toArray();
				for(int j = 0;j < aLocNames.length;j++){
					String aNameLoc = (String)aLocNames[j];
					Alloc aAllocLoc = aAlloc.location.get(aNameLoc);
					out.write(("ALLOCLOC\t" + aNameLoc+"\t"+aAllocLoc.counts+"\t"+aAllocLoc.gcs+"\n").getBytes("iso-8859-1"));
				}

				countDumped++;
			}
			System.out.println("dumped " + countDumped + " allocs");
			out.write("END\n".getBytes("iso-8859-1"));
			out.flush();
		}
	}

	/**
	 * ShutdownHook: This will dump the profiling info when the VM shutsdown.
	 */
	public void run() {
		System.out.println("SHUTDOWN.. ");
	}	

}
