package em.jop.console;

import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.Socket;
import java.util.Comparator;
import java.util.Date;
import java.util.TreeMap;
import java.util.Vector;

import em.jop.Console;

//EM 29/10/2008 : first data/interface separation (really not perfect for the moment)
public class Client {
	static public final int _SERVERPORT = 4444;

	Console console = null;
	
	//EM 12/10/2008
	static final Object sockLock = new Object();
	//EM 23/10/2008 : try to reuse socket
	Socket clientSocket = null;
	BufferedOutputStream outBR = null;
	BufferedReader inBR = null;
	
	//Update data
	public static TreeMap<Long,ThreadStackView> threadMap = new TreeMap<Long,ThreadStackView>();
	//EM 11/10/008
	public int countUpdated = 0;
	//EM 13/10/2008
	public int updateTimePos = 0;

	//EM 28/10/2008 : new call tree
	public MethodView methodsTreeRoot = new MethodView("root");
	//EM 18/10/2008 : tree viewing data
	public MethodView threadsTreeRoot = new MethodView("root");
	public MethodView calleesTreeRoot = new MethodView("root");
	public int showTree = 0;//0=methodsTree, 1=threadsTree, 2=calleesTree, 3=allocs

	public MethodView[] displayMethods = new MethodView[1024];
	public int displayMethodsNb = 0;

	//EM 12/10/2008
	public boolean dispFromMark = true;
	public boolean dispJopTimes = false;
	public int markTimePos = 0;

	//EM 25/10/2008
	public AllocView[] displayAllocs = new AllocView[1024];
	public int displayAllocsNb = 0;
	public Comparator<MethodView> compTimeSpent = new Comparator<MethodView>(){
		public int compare(MethodView m1, MethodView m2) {
			if(dispFromMark){
				if((m1.timeSpent - m1.timeSpentMark) > (m2.timeSpent - m2.timeSpentMark)){
					return -1;
				}
				if((m1.timeSpent - m1.timeSpentMark) < (m2.timeSpent - m2.timeSpentMark)){
					return +1;
				}
			}
			else{
				if(m1.timeSpent > m2.timeSpent){
					return -1;
				}
				if(m1.timeSpent < m2.timeSpent){
					return +1;
				}
			}
			return m1.methodSign.compareTo(m2.methodSign);
		}
	};
	
	//EM 21/10/2008
	//EM 25/10/2008 : use AllocView
	public Comparator<AllocView> compAllocs = new Comparator<AllocView>(){
		public int compare(AllocView a1, AllocView a2) {
			long aCount1 = a1.count - a1.countGC; 
			long aCount2 = a2.count - a2.countGC; 
			if(dispFromMark){
				aCount1 -= a1.countMark - a1.countGCMark;
				aCount2 -= a2.countMark - a2.countGCMark;
			}
			if(aCount2 > aCount1){
				return +1;
			}
			if(aCount2 < aCount1){
				return -1;
			}
			aCount1 = a1.count; 
			aCount2 = a2.count; 
			if(dispFromMark){
				aCount1 -= a1.countMark;
				aCount2 -= a2.countMark;
			}
			if(aCount2 > aCount1){
				return +1;
			}
			if(aCount2 < aCount1){
				return -1;
			}
			return a1.name.compareTo(a2.name);
		}
	};

	//EM 21/10/2008
	//EM 25/10/2008 : use AllocView
	//EM 26/10/2008 : use an AllocView as root
	public AllocView allocsTreeRoot = new AllocView("root");

	public Vector memTot = new Vector();
	public Vector memFree = new Vector();
	//EM 21/10/2008
	public Vector allocsTot = new Vector();
	public Vector gcTot = new Vector();

	public Client(Console aConsole){
		console = aConsole;
	}
	//EM 23/10/2008 : try to reuse socket
	void createSocket() throws Exception {
		System.out.println("NEW SOCKET !");
		clientSocket = new Socket(console.serverIP.getText(), _SERVERPORT);
		outBR = new BufferedOutputStream(clientSocket.getOutputStream(), 1024);
		inBR = new BufferedReader(new InputStreamReader(clientSocket.getInputStream(),"iso-8859-1"),10*1024*1024);
		//EM 28/10/2008
		console.statusL.setText("Connected to JOPed app at " + console.serverIP.getText() + ":" + _SERVERPORT);
	}

	//EM 21/01/2008
	public void sendForceGC(){
		synchronized(sockLock){
			try{
				System.out.println("CLIENT CONN FORCEGC " + new Date().toLocaleString());
				//EM 23/10/2008 : try to reuse socket
				if(clientSocket == null || clientSocket.isClosed()){
					createSocket();
				}
				outBR.write("FORCEGC\n".getBytes("iso-8859-1"));
				outBR.flush();
				//EM 23/10/2008 : try to reuse socket
				//aSocket.close();
			}catch(Throwable t){
				t.printStackTrace(System.err);
				//EM 28/10/2008
				clientSocket = null;
				console.statusL.setText("Can't connect to JOPed app at " + console.serverIP.getText() + ":" + _SERVERPORT);
			}
		}
	}

	public void updateTree(){
		synchronized(sockLock){
			try{
				long aStart = System.nanoTime();
				updateTimePos = memTot.size() - 1;
				System.out.println("CLIENT CONN GET " + new Date().toLocaleString());
				//EM 23/10/2008 : try to reuse socket
				if(clientSocket == null || clientSocket.isClosed()){
					createSocket();
				}
				
				String aLine;
				StringBuffer aSB = new StringBuffer();
				ThreadStackView aThreadStack = null;
				MethodView aThreadMethod = null;
				MethodView aMethod = null; 
				//EM 11/10/008
				countUpdated = 0;

				outBR.write("GETMETHODS\n".getBytes("iso-8859-1"));
				outBR.flush();
				long aCountChars = 0;
				while ((aLine = inBR.readLine()) != null) {
//					System.out.println("CLIENT CONN:" + aLine);
					aCountChars += aLine.length() + 2;
					if ("END".equalsIgnoreCase(aLine)){
						break;
					}
					String[] aToks = aLine.split("\t");
					if(aLine.startsWith("TH")){
						//New thread
						long aThId = Long.parseLong(aToks[1]);
						String aThreadName = aToks[2];
						aThreadStack = threadMap.get(aThId);
						if(aThreadStack == null){
							//Do not exists yet, create a new one
							threadMap.put(aThId, aThreadStack = new ThreadStackView(aThId,aThreadName));
						}
						String aSign = "Thread " + aThId + " '" + aThreadName +"'";
						aThreadMethod = threadsTreeRoot.subMethods.get(aSign);
						if(aThreadMethod == null){
							//No "virtual" method for this thread, create a new one
							aThreadMethod = new MethodView(aSign);
							threadsTreeRoot.subMethods.put(aSign, aThreadMethod);
						}
						//EM 11/10/2008 : there isn't any known time for a thread. Calculated with sub methods values
						aThreadMethod.timeSpent = 0;
						aThreadMethod.count = 0;
						aThreadMethod.timeJopSpent = 0;

						continue;
					}
					if(aLine.startsWith("UP")){
						//End of current method, sort values
						aThreadStack.stackDepth--;
//						aThreadStack.stack[aThreadStack.stackDepth].buildSorted(compTimeSpent);
						continue;
					}
					if(aLine.startsWith("SUB")){
						//Check stack length
						if(aThreadStack.stack.length <= aThreadStack.stackDepth){
							MethodView[] aNewStack = new MethodView[aThreadStack.stack.length * 2];
							for(int i = 0;i < aThreadStack.stack.length;i++){
								aNewStack[i] = aThreadStack.stack[i];
							}
							aThreadStack.stack = aNewStack;
						}

						//EM 11/10/008
						if((++countUpdated % 1000) == 0){
//							System.out.println("Updated " + countUpdated + " calls..");
							console.setTitle("Updated " + countUpdated + " calls..");
						}

						//Method
						String aSign = aToks[1];
						long aTimeSpent = Long.parseLong(aToks[2]);
						long aCount = Long.parseLong(aToks[3]);
						long aTimeJopSpent = Long.parseLong(aToks[4]);
						if(aThreadStack.stackDepth <= 0){
							//Search in thread method
							MethodView aOldM = aThreadMethod.subMethods.get(aSign);
							if(aOldM != null){
								//Simple update
								aOldM.timeSpent = aTimeSpent;
								aOldM.count = aCount;
								aOldM.timeJopSpent = aTimeJopSpent;
								aThreadStack.stack[aThreadStack.stackDepth] = aOldM;
								aThreadStack.stackDepth++;
								continue;
							}
							//Add new first method of the stack
							aMethod = new MethodView(aSign);
							aMethod.timeSpent = aTimeSpent;
							aMethod.count = aCount;
							aMethod.timeJopSpent = aTimeJopSpent;
							aThreadStack.stack[aThreadStack.stackDepth] = aMethod;
							aThreadStack.stackDepth++;
							aThreadMethod.subMethods.put(aSign, aMethod);
							continue;
						}
						//Search in current method
						MethodView aOldM = aThreadStack.stack[aThreadStack.stackDepth - 1].subMethods.get(aSign);
						if(aOldM != null){
							//Simple update
							aOldM.timeSpent = aTimeSpent;
							aOldM.count = aCount;
							aOldM.timeJopSpent = aTimeJopSpent;
							aThreadStack.stack[aThreadStack.stackDepth] = aOldM;
							aThreadStack.stackDepth++;
							continue;
						}
						//Add new method
						aMethod = new MethodView(aSign);
						aThreadStack.stack[aThreadStack.stackDepth - 1].subMethods.put(aSign, aMethod);
						aMethod.timeSpent = aTimeSpent;
						aMethod.count = aCount;
						aMethod.timeJopSpent = aTimeJopSpent;
						aThreadStack.stack[aThreadStack.stackDepth] = aMethod;
						aThreadStack.stackDepth++;
						continue;
					}
				}
				System.out.println("CLIENT CONN GETMETHODS DONE " + (aCountChars / 1024) + "K " + console.formatTime(System.nanoTime() - aStart));

				//EM 21/10/2008
				outBR.write("GETALLOCS\n".getBytes("iso-8859-1"));
				outBR.flush();
				aCountChars=0;
				Long aZero = new Long(0);
				AllocView aAlloc = null;
				while ((aLine = inBR.readLine()) != null) {
//					System.out.println("CLIENT CONN:" + aLine);
					aCountChars += aLine.length() + 2;
					if ("END".equalsIgnoreCase(aLine)){
						break;
					}
					String[] aToks = aLine.split("\t");
					if("ALLOC".equals(aToks[0])){
						//EM 26/10/2008 : use allocsTreeRoot
						aAlloc = allocsTreeRoot.location.get(aToks[1]);
						if(aAlloc == null){
							aAlloc = new AllocView(aToks[1]);
							allocsTreeRoot.location.put(aToks[1], aAlloc);
						}
						aAlloc.count = Long.parseLong(aToks[2]);
						aAlloc.countGC = Long.parseLong(aToks[3]);
						continue;
					}
					if("ALLOCLOC".equals(aToks[0])){
						AllocView aAllocLoc = aAlloc.location.get(aToks[1]);
						if(aAllocLoc == null){
							aAllocLoc = new AllocView(aToks[1]);
							aAlloc.location.put(aToks[1], aAllocLoc);
						}
						aAllocLoc.count = Long.parseLong(aToks[2]);
						aAllocLoc.countGC = Long.parseLong(aToks[3]);
					}
				}
				System.out.println("CLIENT CONN GETALLOCS DONE " + (aCountChars / 1024) + "K " + console.formatTime(System.nanoTime() - aStart));
				
				//EM 23/10/2008 : try to reuse socket
				//aSocket.close();

				//Final sorting
//				Object[] aMs = rootMethod.subMethods.values().toArray();
//				for(int m = 0;m < aMs.length;m++){
//				((Method)aMs[m]).buildSorted(compTimeSpent);
//				}
				console.setTitle("Sorting..");
				threadsTreeRoot.calcSortRecurse(compTimeSpent);
				//EM 18/10/2008 : as the callee tree is not fully expended, re-create it from scratch (need a better update to keep expend properties)
				threadsTreeRoot.buildCallee(calleesTreeRoot = new MethodView("root"),compTimeSpent);
				//EM 26/10/2008
				allocsTreeRoot.sortRecurse(compAllocs);
				//EM 28/10/2008
				methodsTreeRoot.initRecurse();
				Object[] aMThreads = threadsTreeRoot.subMethods.values().toArray();
				for(int mt = 0;mt < aMThreads.length;mt++){
					MethodView aMThread = (MethodView)aMThreads[mt];
					Object[] aMs = aMThread.subMethods.values().toArray();
					for(int m = 0;m < aMs.length;m++){
						MethodView aM = (MethodView)aMs[m];
						methodsTreeRoot.addRecurse(aM);
					}
				}
				methodsTreeRoot.sortRecurse(compTimeSpent);

				
				//Update display
				console.statusL.setText(countUpdated + " methods and " + threadsTreeRoot.subMethodsSorted.size() + " threads in thread tree, " + allocsTreeRoot.locationSorted.size() + " classes in alloc tree");
				console.setTitle(console._TITLE);
			}catch(Throwable t){
				t.printStackTrace(System.err);
				//EM 28/10/2008
				clientSocket = null;
				console.statusL.setText("Can't connect to JOPed app at " + console.serverIP.getText() + ":" + _SERVERPORT);
			}
		}
	}

	//EM 12/10/2008
	public void setMark(){
		markTimePos = (updateTimePos >= 0) ? updateTimePos : 0;
		methodsTreeRoot.setMark();
		threadsTreeRoot.setMark();
		calleesTreeRoot.setMark();
		//EM 26/10/2008 ; use allocsTreeRoot
		allocsTreeRoot.setMark();
	}

	//EM 12/10/2008
	public void trackMem(){
		synchronized(sockLock){
			try{
				try{
					//EM 23/10/2008 : try to reuse socket
					if(clientSocket == null || clientSocket.isClosed()){
						createSocket();
					}
				}
				catch(Throwable t){
					//Can't connect
					//EM 28/10/2008
					clientSocket = null;
					console.statusL.setText("Can't connect to JOPed app at " + console.serverIP.getText() + ":" + _SERVERPORT);
					return;
				}
				BufferedWriter outBR = new BufferedWriter(new OutputStreamWriter(clientSocket.getOutputStream()), 1024);
				BufferedReader in = new BufferedReader(new InputStreamReader(clientSocket.getInputStream()),1024);
				outBR.write("GETMEM\n");
				outBR.flush();
				String aLine;
				StringBuffer aSB = new StringBuffer();
				ThreadStackView aThreadStack = null;
				MethodView aThreadMethod = null;
				MethodView aMethod = null; 
				//EM 11/10/008
				countUpdated = 0;

				if((aLine = in.readLine()) != null) {
					//System.out.println(aLine);
					String[] aToks = aLine.split("\t");
					memTot.add(Long.parseLong(aToks[1]));
					memFree.add(Long.parseLong(aToks[2]));
					allocsTot.add(Long.parseLong(aToks[3]));
					gcTot.add(Long.parseLong(aToks[4]));
				}
				//EM 23/10/2008 : try to reuse socket
				//aSocket.close();
			}catch(Throwable t){
				t.printStackTrace(System.err);
				//EM 28/10/2008
				clientSocket = null;
				console.statusL.setText("Can't connect to JOPed app at " + console.serverIP.getText() + ":" + _SERVERPORT);
			}
		}
	}

}
