package em.jop.profiler;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.ServerSocket;
import java.net.Socket;

import em.jop.Profiler;

public class Server {

	ServerSocket serverSocket;

	public Server(){
		try {
			serverSocket = new ServerSocket(4444);
		}catch(Throwable t){
			t.printStackTrace(System.err);
		}
		Thread aTh = new Thread(new Runnable(){
			public void run() {
				while(true){
					//System.out.println("JOP SERVER WAITING.. "+new Date().toLocaleString());
					Socket aSocket = null;
					try {
						aSocket = serverSocket.accept();
					}catch(Throwable t){
						t.printStackTrace(System.err);
					}
					handleSocket(aSocket);
				}
			}			
		},"JOP SERVER");
		aTh.start();
	}

	void handleSocket(final Socket aSocket){
		//System.out.println("JOP SERVER CONN IN "+new Date().toLocaleString());
		try{
			Thread aTh = new Thread(new Runnable(){
				public void run() {
					handleConn(aSocket);
				}
			},"SERVER CONN");
			aTh.start();
		}catch(Throwable t){
			t.printStackTrace(System.err);
		}
		//System.out.println("JOP SERVER CONN DONE "+new Date().toLocaleString());
	}

	void handleConn(Socket aSocket){
		try{
			BufferedReader in = new BufferedReader(new InputStreamReader(aSocket.getInputStream(),"iso-8859-1"),1024);
			byte[] aBuf = new byte[1024];
			String aLine;
			while((aLine = in.readLine()) != null){
				System.out.println("JOP SERVER CONN:" + aLine);
				long aStart = System.nanoTime();
				if("GETMETHODS".equalsIgnoreCase(aLine)){
					//Ok, send
					BufferedOutputStream out = new BufferedOutputStream(aSocket.getOutputStream(),10*1024*1024);
					Profiler.dump(out);
				}
				else if("GETALLOCS".equalsIgnoreCase(aLine)){
					//Ok, send
					BufferedOutputStream out = new BufferedOutputStream(aSocket.getOutputStream(),10*1024);
					Profiler.dumpAllocs(out);
				}
				else if("GETMEM".equalsIgnoreCase(aLine)){
					BufferedWriter out = new BufferedWriter(new OutputStreamWriter(aSocket.getOutputStream()),1024);
					Runtime aR = Runtime.getRuntime();
					out.write(aR.maxMemory() + "\t" + aR.totalMemory() + "\t" + aR.freeMemory()+"\t"+Profiler.countAllocs+"\t"+Profiler.countGC+"\n");
					out.flush();
				}
				else if("FORCEGC".equalsIgnoreCase(aLine)){
					System.gc();
					System.out.println("JOP SERVER FORCEGC DONE");
				}
				System.out.println("JOP SERVER CONN DONE "+formatTime(System.nanoTime() - aStart));
			}
		}catch(Throwable t){
			t.printStackTrace(System.err);
		}
	}
	
	String formatTime(long aT){
		long aMStot = aT / 1000000L;
		long aStot = aMStot / 1000;
		long aMtot = aStot/60;
		long aHtot = aMtot/60;
		if(aHtot > 0){
			return aHtot + "h" + (aMtot % 60) + "m";
		}
		if(aMtot > 0){
			return (aMtot % 60) + "m" + (aStot % 60) + "s";
		}
		return (aStot % 60) + "s" + (aMStot % 1000) + "ms";
	}

}
